from PodSix.Resource import *
from Widget import Widget

from pygame import draw, image
from pygame.locals import *
import time

class TextInput(Widget):
	def __init__(self, pos={}, width=0.7, text="", mapping={}, font="default", color=[0, 0, 0], onhover=False):
		Widget.__init__(self)
		self.text = text
		self.font = font
		self.color = color
		self.down = False
		self.rect = None
		self.pos = pos
		self.cursor_pos = len(text)
		self.gotfocus = False
		self.width = width
		self.mapping = mapping
		self.onhover = onhover
		# special keys we want to ignore
		self.special_keys = range(0, 10) + range(11, 32) + range(127, 160) + range(0x111, 0x211)
		
		self.cursor_timer = 0
		self.cursor_is_visible = True
		
		self.letterWidth, self.letterHeight = gfx.TextSize("A")
	
	def KeyDown(self, e):
		self.Draw()
		if not self.onhover or self.gotfocus:
			#if self.textWidth < self.width * gfx.width  - self.letterWidth and e.key in self.mapping:
			#    self.text = self.text[:self.cursor_pos] + self.mapping[e.key] + self.text[self.cursor_pos:]
			if self.textWidth < self.width * gfx.width  - self.letterWidth and e.key not in self.special_keys:
				self.text = self.text[:self.cursor_pos] + e.unicode + self.text[self.cursor_pos:]
				self.cursor_pos += 1
			self.triggered = True
	
	def MouseDown(self, e):
		if self.rect and self.InRect(e.pos):
			self.down = True
		self.SetFocus(self.down)
	
	def MouseUp(self, e):
		if self.down:
			self.down = False
	
	def KeyDown_backspace(self, e):
		if not self.onhover or self.gotfocus:
			if self.cursor_pos > 0:
			    self.text = self.text[:self.cursor_pos-1] + self.text[self.cursor_pos:]
			    self.cursor_pos -= 1
			
	def KeyDown_delete(self, e):
		if not self.onhover or self.gotfocus:
			if self.cursor_pos < len(self.text):
			    self.text = self.text[:self.cursor_pos] + self.text[self.cursor_pos+1:]
	
	# cursor movement		
	def KeyDown_left(self, e):
		if not self.onhover or self.gotfocus:
			if self.cursor_pos > 0:
				self.cursor_pos -= 1
				
	def KeyDown_right(self, e):
		if not self.onhover or self.gotfocus:
			if self.cursor_pos < len(self.text):
				self.cursor_pos += 1
	
	def KeyDown_home(self, e):
		if not self.onhover or self.gotfocus:
			self.cursor_pos = 0
			
	def KeyDown_end(self, e):
		if not self.onhover or self.gotfocus:
			self.cursor_pos = len(self.text)
			
	
	def SetFocus(self, focus):
		self.gotfocus = focus
	# needs better focus check
#		if not self.onhover or self.gotfocus:
#			pygame.key.set_repeat(50,50)
#		else:
#			pygame.key.set_repeat()
	
	def Draw(self):
		# cursor blinking update stuff; move to an update method?
		self.cursor_timer -= self.Elapsed()
		if self.cursor_timer <= 0:
		    self.cursor_is_visible = not self.cursor_is_visible
		    self.cursor_timer = 400/ 1000.
		
		self.rect = gfx.DrawText(self.text, self.pos, self.color, self.font)
		self.textWidth = self.rect.width
		# is there a wrapper in gfx for that? (gfx.TextSize is sth differen, isn't it?)
		width_to_cursor = gfx.font[self.font]["font"].size(self.text[:self.cursor_pos])[0]
		cursor = width_to_cursor + self.rect.left
		self.rect.width = gfx.width * self.width
		gfx.AlignToPos(self.rect, self.pos)
		# why don't you just inflate/adjust a rect and draw it with one call? or whtats this?
		gfx.DrawLines([(self.rect.left - 3, self.rect.top - 4), (self.rect.right + 4, self.rect.top - 4)], self.color, False)
		gfx.DrawLines([(self.rect.left - 3, self.rect.bottom + 4), (self.rect.right + 4, self.rect.bottom + 4)], self.color, False)
		gfx.DrawLines([(self.rect.left - 4, self.rect.top - 3), (self.rect.left - 4, self.rect.bottom + 4)], self.color, False)
		gfx.DrawLines([(self.rect.right + 4, self.rect.top - 3), (self.rect.right + 4, self.rect.bottom + 4)], self.color, False)
		
		if not self.onhover or self.gotfocus:
			if self.cursor_is_visible:
			    gfx.DrawLines([(cursor, self.rect.top), (cursor, self.rect.bottom)], self.color, False)

